@@ -36,19 +36,23 @@
 #define ANIM_CLASS_BIT_TITLE_INITIAL	0
 #define ANIM_CLASS_BIT_TITLE		1
 #define ANIM_CLASS_BIT_MAIN		2
-#define ANIM_CLASS_BIT_SCORES		3
-#define ANIM_CLASS_BIT_SUBMENU		4
-#define ANIM_CLASS_BIT_MENU		5
-#define ANIM_CLASS_BIT_TOONS		6
-#define ANIM_CLASS_BIT_NO_TITLE		7
+#define ANIM_CLASS_BIT_NAMES		3
+#define ANIM_CLASS_BIT_SCORES		4
+#define ANIM_CLASS_BIT_SCORESONLY	5
+#define ANIM_CLASS_BIT_SUBMENU		6
+#define ANIM_CLASS_BIT_MENU		7
+#define ANIM_CLASS_BIT_TOONS		8
+#define ANIM_CLASS_BIT_NO_TITLE		9
 
-#define NUM_ANIM_CLASSES		8
+#define NUM_ANIM_CLASSES		10
 
 #define ANIM_CLASS_NONE			0
 #define ANIM_CLASS_TITLE_INITIAL	(1 << ANIM_CLASS_BIT_TITLE_INITIAL)
 #define ANIM_CLASS_TITLE		(1 << ANIM_CLASS_BIT_TITLE)
 #define ANIM_CLASS_MAIN			(1 << ANIM_CLASS_BIT_MAIN)
+#define ANIM_CLASS_NAMES		(1 << ANIM_CLASS_BIT_NAMES)
 #define ANIM_CLASS_SCORES		(1 << ANIM_CLASS_BIT_SCORES)
+#define ANIM_CLASS_SCORESONLY		(1 << ANIM_CLASS_BIT_SCORESONLY)
 #define ANIM_CLASS_SUBMENU		(1 << ANIM_CLASS_BIT_SUBMENU)
 #define ANIM_CLASS_MENU			(1 << ANIM_CLASS_BIT_MENU)
 #define ANIM_CLASS_TOONS		(1 << ANIM_CLASS_BIT_TOONS)
@@ -58,6 +62,11 @@
 					 ANIM_CLASS_SCORES	|	\
 					 ANIM_CLASS_NO_TITLE)
 
+#define ANIM_CLASS_TOONS_SCORESONLY	(ANIM_CLASS_TOONS 	|	\
+					 ANIM_CLASS_SCORES	|	\
+					 ANIM_CLASS_SCORESONLY	|	\
+					 ANIM_CLASS_NO_TITLE)
+
 #define ANIM_CLASS_TOONS_MENU_MAIN	(ANIM_CLASS_TOONS 	|	\
 					 ANIM_CLASS_MENU  	|	\
 					 ANIM_CLASS_MAIN  	|	\
@@ -68,6 +77,12 @@
 					 ANIM_CLASS_SUBMENU	|	\
 					 ANIM_CLASS_NO_TITLE)
 
+#define ANIM_CLASS_TOONS_MENU_SUBMENU_2	(ANIM_CLASS_TOONS 	|	\
+					 ANIM_CLASS_MENU  	|	\
+					 ANIM_CLASS_SUBMENU	|	\
+					 ANIM_CLASS_NAMES 	|	\
+					 ANIM_CLASS_NO_TITLE)
+
 // values for global animation states
 #define ANIM_STATE_INACTIVE		0
 #define ANIM_STATE_RESTART		(1 << 0)
@@ -109,7 +124,8 @@ struct GlobalAnimPartControlInfo
 
   unsigned int initial_anim_sync_frame;
   unsigned int anim_random_frame;
-  unsigned int step_delay, step_delay_value;
+
+  DelayCounter step_delay;
 
   int init_delay_counter;
   int anim_delay_counter;
@@ -183,12 +199,13 @@ struct GameModeAnimClass
   { GAME_MODE_LEVELNR,			ANIM_CLASS_TOONS_MENU_SUBMENU	},
   { GAME_MODE_INFO,			ANIM_CLASS_TOONS_MENU_SUBMENU	},
   { GAME_MODE_SETUP,			ANIM_CLASS_TOONS_MENU_SUBMENU	},
-  { GAME_MODE_PSEUDO_NAMESONLY,		ANIM_CLASS_TOONS_MENU_SUBMENU	},
-  { GAME_MODE_PSEUDO_TYPENAMES,		ANIM_CLASS_TOONS_MENU_SUBMENU	},
+  { GAME_MODE_PSEUDO_NAMESONLY,		ANIM_CLASS_TOONS_MENU_SUBMENU_2	},
+  { GAME_MODE_PSEUDO_TYPENAMES,		ANIM_CLASS_TOONS_MENU_SUBMENU_2	},
   { GAME_MODE_PSEUDO_MAINONLY,		ANIM_CLASS_TOONS_MENU_MAIN	},
   { GAME_MODE_PSEUDO_TYPENAME,		ANIM_CLASS_TOONS_MENU_MAIN	},
-  { GAME_MODE_PSEUDO_SCORESOLD,		ANIM_CLASS_TOONS_SCORES		},
-  { GAME_MODE_PSEUDO_SCORESNEW,		ANIM_CLASS_TOONS_SCORES		},
+  { GAME_MODE_PSEUDO_SCORESOLD,		ANIM_CLASS_TOONS_SCORESONLY	},
+  { GAME_MODE_PSEUDO_SCORESNEW,		ANIM_CLASS_TOONS_SCORESONLY	},
+  { GAME_MODE_SCOREINFO,		ANIM_CLASS_TOONS_SCORES		},
   { GAME_MODE_EDITOR,			ANIM_CLASS_NO_TITLE		},
   { GAME_MODE_PLAYING,			ANIM_CLASS_NO_TITLE		},
 
@@ -204,7 +221,9 @@ struct AnimClassGameMode
   { ANIM_CLASS_BIT_TITLE_INITIAL,	GAME_MODE_TITLE_INITIAL		},
   { ANIM_CLASS_BIT_TITLE,		GAME_MODE_TITLE			},
   { ANIM_CLASS_BIT_MAIN,		GAME_MODE_MAIN			},
+  { ANIM_CLASS_BIT_NAMES,		GAME_MODE_NAMES			},
   { ANIM_CLASS_BIT_SCORES,		GAME_MODE_SCORES		},
+  { ANIM_CLASS_BIT_SCORESONLY,		GAME_MODE_PSEUDO_SCORESONLY	},
   { ANIM_CLASS_BIT_SUBMENU,		GAME_MODE_PSEUDO_SUBMENU	},
   { ANIM_CLASS_BIT_MENU,		GAME_MODE_PSEUDO_MENU		},
   { ANIM_CLASS_BIT_TOONS,		GAME_MODE_PSEUDO_TOONS		},
@@ -392,7 +411,8 @@ static void InitToonControls(void)
     int sound = SND_UNDEFINED;
     int music = MUS_UNDEFINED;
     int graphic = IMG_TOON_1 + i;
-    int control = graphic;
+
+    control = graphic;
 
     part->nr = part_nr;
     part->anim_nr = anim_nr;
@@ -418,8 +438,8 @@ static void InitToonControls(void)
     part->initial_anim_sync_frame = 0;
     part->anim_random_frame = -1;
 
-    part->step_delay = 0;
-    part->step_delay_value = graphic_info[control].step_delay;
+    part->step_delay.count = 0;
+    part->step_delay.value = graphic_info[control].step_delay;
 
     part->state = ANIM_STATE_INACTIVE;
     part->last_anim_status = -1;
@@ -525,8 +545,8 @@ static void InitGlobalAnimControls(void)
 	part->initial_anim_sync_frame = 0;
 	part->anim_random_frame = -1;
 
-	part->step_delay = 0;
-	part->step_delay_value = graphic_info[control].step_delay;
+	part->step_delay.count = 0;
+	part->step_delay.value = graphic_info[control].step_delay;
 
 	part->state = ANIM_STATE_INACTIVE;
 	part->last_anim_status = -1;
@@ -616,11 +636,77 @@ void InitGlobalAnimations(void)
   InitGlobalAnimControls();
 }
 
-static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
+static void BlitGlobalAnimation(struct GlobalAnimPartControlInfo *part,
+				Bitmap *src_bitmap, int src_x0, int src_y0,
+				int drawing_target)
 {
+  struct GraphicInfo *g = &part->graphic_info;
+  struct GraphicInfo *c = &part->control_info;
+  void (*blit_bitmap)(Bitmap *, Bitmap *, int, int, int, int, int, int) =
+    (g->draw_masked ? BlitBitmapMasked : BlitBitmap);
+  void (*blit_screen)(Bitmap *, int, int, int, int, int, int) =
+    (g->draw_masked ? BlitToScreenMasked : BlitToScreen);
   Bitmap *fade_bitmap =
     (drawing_target == DRAW_TO_FADE_SOURCE ? gfx.fade_bitmap_source :
      drawing_target == DRAW_TO_FADE_TARGET ? gfx.fade_bitmap_target : NULL);
+  int x, y;
+
+  for (y = 0; y < c->stacked_yfactor; y++)
+  {
+    for (x = 0; x < c->stacked_xfactor; x++)
+    {
+      int src_x = src_x0;
+      int src_y = src_y0;
+      int dst_x = part->x + x * (g->width  + c->stacked_xoffset);
+      int dst_y = part->y + y * (g->height + c->stacked_yoffset);
+      int cut_x = 0;
+      int cut_y = 0;
+      int width  = g->width;
+      int height = g->height;
+
+      if (dst_x < 0)
+      {
+	width += dst_x;
+	cut_x = -dst_x;
+	dst_x = 0;
+      }
+      else if (dst_x > part->viewport_width - g->width)
+      {
+	width -= (dst_x - (part->viewport_width - g->width));
+      }
+
+      if (dst_y < 0)
+      {
+	height += dst_y;
+	cut_y  = -dst_y;
+	dst_y = 0;
+      }
+      else if (dst_y > part->viewport_height - g->height)
+      {
+	height -= (dst_y - (part->viewport_height - g->height));
+      }
+
+      if (width <= 0 || height <= 0)
+	continue;
+
+      src_x += cut_x;
+      src_y += cut_y;
+
+      dst_x += part->viewport_x;
+      dst_y += part->viewport_y;
+
+      if (drawing_target == DRAW_TO_SCREEN)
+	blit_screen(src_bitmap, src_x, src_y, width, height,
+		    dst_x, dst_y);
+      else
+	blit_bitmap(src_bitmap, fade_bitmap, src_x, src_y, width, height,
+		    dst_x, dst_y);
+    }
+  }
+}
+
+static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
+{
   int game_mode_anim_action[NUM_GAME_MODES];
   int mode_nr;
 
@@ -765,18 +851,8 @@ static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
 	struct GraphicInfo *g = &part->graphic_info;
 	Bitmap *src_bitmap;
 	int src_x, src_y;
-	int width  = g->width;
-	int height = g->height;
-	int dst_x = part->x;
-	int dst_y = part->y;
-	int cut_x = 0;
-	int cut_y = 0;
 	int sync_frame;
 	int frame;
-	void (*blit_bitmap)(Bitmap *, Bitmap *, int, int, int, int, int, int) =
-	  (g->draw_masked ? BlitBitmapMasked : BlitBitmap);
-	void (*blit_screen)(Bitmap *, int, int, int, int, int, int) =
-	  (g->draw_masked ? BlitToScreenMasked : BlitToScreen);
 	int last_anim_random_frame = gfx.anim_random_frame;
 
 	if (!(part->state & ANIM_STATE_RUNNING))
@@ -785,30 +861,6 @@ static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
 	if (part->drawing_stage != drawing_stage)
 	  continue;
 
-	if (part->x < 0)
-	{
-	  dst_x = 0;
-	  width += part->x;
-	  cut_x = -part->x;
-	}
-	else if (part->x > part->viewport_width - g->width)
-	  width -= (part->x - (part->viewport_width - g->width));
-
-	if (part->y < 0)
-	{
-	  dst_y = 0;
-	  height += part->y;
-	  cut_y = -part->y;
-	}
-	else if (part->y > part->viewport_height - g->height)
-	  height -= (part->y - (part->viewport_height - g->height));
-
-	if (width <= 0 || height <= 0)
-	  continue;
-
-	dst_x += part->viewport_x;
-	dst_y += part->viewport_y;
-
 	sync_frame = anim_sync_frame - part->initial_anim_sync_frame;
 
 	// re-initialize random animation frame after animation delay
@@ -828,15 +880,7 @@ static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
 	getGlobalAnimGraphicSource(part->graphic, frame, &src_bitmap,
 				   &src_x, &src_y);
 
-	src_x += cut_x;
-	src_y += cut_y;
-
-	if (drawing_target == DRAW_TO_SCREEN)
-	  blit_screen(src_bitmap, src_x, src_y, width, height,
-		      dst_x, dst_y);
-	else
-	  blit_bitmap(src_bitmap, fade_bitmap, src_x, src_y, width, height,
-		      dst_x, dst_y);
+	BlitGlobalAnimation(part, src_bitmap, src_x, src_y, drawing_target);
       }
     }
   }
@@ -1125,44 +1169,64 @@ static boolean isClickablePart(struct GlobalAnimPartControlInfo *part, int mask)
   return FALSE;
 }
 
-static boolean isClickedPart(struct GlobalAnimPartControlInfo *part,
-			     int mx, int my, boolean clicked)
+static boolean isInsidePartStacked(struct GlobalAnimPartControlInfo *part,
+				   int mx, int my)
 {
   struct GraphicInfo *g = &part->graphic_info;
+  struct GraphicInfo *c = &part->control_info;
   int part_x = part->viewport_x + part->x;
   int part_y = part->viewport_y + part->y;
   int part_width  = g->width;
   int part_height = g->height;
+  int x, y;
 
+  for (y = 0; y < c->stacked_yfactor; y++)
+  {
+    for (x = 0; x < c->stacked_xfactor; x++)
+    {
+      int part_stacked_x = part_x + x * (part_width  + c->stacked_xoffset);
+      int part_stacked_y = part_y + y * (part_height + c->stacked_yoffset);
+
+      if (mx >= part_stacked_x &&
+	  mx <  part_stacked_x + part_width &&
+	  my >= part_stacked_y &&
+	  my <  part_stacked_y + part_height)
+	return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+static boolean isClickedPart(struct GlobalAnimPartControlInfo *part,
+			     int mx, int my, boolean clicked)
+{
   // check if mouse click was detected at all
   if (!clicked)
     return FALSE;
 
-  // check if mouse click is inside the animation part's viewport
+  // check if mouse click is outside the animation part's viewport
   if (mx <  part->viewport_x ||
       mx >= part->viewport_x + part->viewport_width ||
       my <  part->viewport_y ||
       my >= part->viewport_y + part->viewport_height)
     return FALSE;
 
-  // check if mouse click is inside the animation part's graphic
-  if (mx <  part_x ||
-      mx >= part_x + part_width ||
-      my <  part_y ||
-      my >= part_y + part_height)
-    return FALSE;
+  // check if mouse click is inside the animation part's (stacked) graphic
+  if (isInsidePartStacked(part, mx, my))
+    return TRUE;
 
-  return TRUE;
+  return FALSE;
 }
 
 static boolean clickBlocked(struct GlobalAnimPartControlInfo *part)
 {
-  return (part->control_info.style & STYLE_BLOCK ? TRUE : FALSE);
+  return ((part->control_info.style & STYLE_BLOCK) ? TRUE : FALSE);
 }
 
 static boolean clickConsumed(struct GlobalAnimPartControlInfo *part)
 {
-  return (part->control_info.style & STYLE_PASSTHROUGH ? FALSE : TRUE);
+  return ((part->control_info.style & STYLE_PASSTHROUGH) ? FALSE : TRUE);
 }
 
 static void InitGlobalAnim_Triggered(struct GlobalAnimPartControlInfo *part,
@@ -1295,7 +1359,8 @@ static int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part,
     part->anim_event_state = (c->anim_event != ANIM_EVENT_UNDEFINED);
 
     part->initial_anim_sync_frame =
-      (g->anim_global_sync ? 0 : anim_sync_frame + part->init_delay_counter);
+      (g->anim_global_sync || g->anim_global_anim_sync ? 0 :
+       anim_sync_frame + part->init_delay_counter);
 
     // do not re-initialize random animation frame after fade-in
     if (part->anim_random_frame == -1)
@@ -1513,8 +1578,7 @@ static int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part,
   // special case to prevent expiring loop sounds when playing
   PlayGlobalAnimSoundIfLoop(part);
 
-  if (!DelayReachedExt(&part->step_delay, part->step_delay_value,
-		       anim_sync_frame))
+  if (!DelayReachedExt(&part->step_delay, anim_sync_frame))
     return ANIM_STATE_RUNNING;
 
 #if 0
@@ -1757,7 +1821,10 @@ static boolean DoGlobalAnim_EventAction(struct GlobalAnimPartControlInfo *part)
   if (event_action == ANIM_EVENT_ACTION_NONE)
     return FALSE;
 
-  PushUserEvent(USEREVENT_ANIM_EVENT_ACTION, event_action, 0);
+  if (event_action < MAX_IMAGE_FILES)
+    PushUserEvent(USEREVENT_ANIM_EVENT_ACTION, event_action, 0);
+  else
+    OpenURLFromHash(anim_url_hash, event_action);
 
   // check if further actions are allowed to be executed
   if (part->control_info.style & STYLE_MULTIPLE_ACTIONS)
@@ -1930,6 +1997,18 @@ static void ResetGlobalAnim_Clicked(void)
   InitGlobalAnim_Clicked(-1, -1, ANIM_CLICKED_RESET);
 }
 
+void RestartGlobalAnimsByStatus(int status)
+{
+  int anim_status_last = global.anim_status;
+
+  global.anim_status = status;
+
+  // force restarting global animations by changed global animation status
+  SDLRedrawWindow();
+
+  global.anim_status = anim_status_last;
+}
+
 boolean HandleGlobalAnimClicks(int mx, int my, int button, boolean force_click)
 {
   static boolean click_consumed = FALSE;
@@ -1960,3 +2039,8 @@ boolean HandleGlobalAnimClicks(int mx, int my, int button, boolean force_click)
 
   return click_consumed_current;
 }
+
+int getGlobalAnimSyncFrame(void)
+{
+  return anim_sync_frame;
+}
