@@ -36,7 +36,7 @@ int getScreenFieldSizeY(void);
 
 void PlayLevelSound_EM(int, int, int, int);
 void InitGraphicInfo_EM(void);
-boolean CheckSingleStepMode_EM(byte action[], int, boolean, boolean, boolean);
+boolean CheckSingleStepMode_EM(int, boolean, boolean, boolean);
 
 void SetGfxAnimation_EM(struct GraphicInfo_EM *, int, int, int, int);
 void getGraphicSourceObjectExt_EM(struct GraphicInfo_EM *, int, int, int, int);
@@ -49,7 +49,7 @@ void getGraphicSourcePlayerExt_EM(struct GraphicInfo_EM *, int, int, int);
 
 void CheckSingleStepMode_SP(boolean, boolean);
 
-void getGraphicSource_SP(struct GraphicInfo_SP *, int, int, int, int);
+void getGraphicSource_SP(struct GraphicInfo_SP *, int, int);
 int getGraphicInfo_Delay(int);
 boolean isNextAnimationFrame_SP(int, int);
 
@@ -61,13 +61,22 @@ boolean isNextAnimationFrame_SP(int, int);
 void SetDrawtoField(int);
 
 int el2img_mm(int);
+int el_act2img_mm(int, int);
 
 void CheckSingleStepMode_MM(boolean, boolean);
+void ShowEnvelope_MM(int);
 
 int getGraphicAnimationFrame(int, int);
+int getGraphicAnimationFrameXY(int, int, int);
+
 void getGraphicSource(int, int, Bitmap **, int *, int *);
 void getMiniGraphicSource(int, Bitmap **, int *, int *);
 void getSizedGraphicSource(int, int, int, Bitmap **, int *, int *);
+boolean getGraphicInfo_NewFrame(int, int, int);
 
+void AdvanceFrameCounter(void);
+void AdvanceGfxFrame(void);
+
+int getAnimationFrame(int, int, int, int, int);
 
 #endif	// ENGINES_H
