@@ -63,23 +63,17 @@ void DrawGraphicAnimation_MM(int, int, int, int);
 
 void DrawGraphic_MM(int, int, int);
 void DrawGraphicExt_MM(DrawBuffer *, int, int, int);
-void DrawGraphicThruMask_MM(int, int, int);
-void DrawGraphicThruMaskExt_MM(DrawBuffer *, int, int, int);
+void DrawGraphicThruMask_MM(int, int, int, int);
+void DrawGraphicThruMaskExt_MM(DrawBuffer *, int, int, int, int);
 void DrawMiniGraphic_MM(int, int, int);
 void getMiniGraphicSource(int, Bitmap **, int *, int *);
 void DrawMiniGraphicExt_MM(DrawBuffer *, int, int, int);
 void DrawGraphicShifted_MM(int, int, int, int, int, int, int);
-void DrawGraphicShiftedThruMask_MM(int, int, int, int, int, int);
 void DrawScreenElementExt_MM(int, int, int, int, int, int, int);
 void DrawLevelElementExt_MM(int, int, int, int, int, int, int);
 void DrawScreenElementShifted_MM(int, int, int, int, int, int);
-void DrawLevelElementShifted_MM(int, int, int, int, int, int);
-void DrawScreenElementThruMask_MM(int, int, int);
-void DrawLevelElementThruMask_MM(int, int, int);
-void DrawLevelFieldThruMask_MM(int, int);
 void ErdreichAnbroeckeln(int, int);
 void DrawScreenElement_MM(int, int, int);
-void DrawLevelElement_MM(int, int, int);
 void DrawScreenField_MM(int, int);
 void DrawLevelField_MM(int, int);
 void DrawMiniElement_MM(int, int, int);
@@ -92,7 +86,6 @@ void DrawElement_MM(int, int, int);
 void DrawWallsExt_MM(int, int, int, int);
 void DrawWalls_MM(int, int, int);
 void DrawWallsAnimation_MM(int, int, int, int, int);
-void DrawMiniLevel_MM(int, int, int, int);
 void DrawMicroLevel_MM(int, int, boolean);
 void DrawTileCursor_MM(int, boolean);
 
@@ -114,5 +107,6 @@ int get_num_elements(int);
 int get_rotated_element(int, int);
 
 int el2gfx(int);
+int el_act2gfx(int, int);
 
 #endif
