@@ -139,13 +139,6 @@ char *getTokenFromImageID(int graphic)
   return (file_list != NULL ? file_list->token : NULL);
 }
 
-char *getFilenameFromImageID(int graphic)
-{
-  struct FileInfo *file_list = getImageListEntryFromImageID(graphic);
-
-  return (file_list != NULL ? file_list->filename : NULL);
-}
-
 int getImageIDFromToken(char *token)
 {
   struct FileInfo *file_list = image_info->file_list;
@@ -398,9 +391,11 @@ void ScaleImage(int pos, int zoom_factor)
     return;
 
   if (zoom_factor != 1)
+  {
     ScaleBitmap(img_info->bitmaps, zoom_factor);
 
-  img_info->scaled_up = TRUE;
+    img_info->scaled_up = TRUE;
+  }
 }
 
 void FreeAllImages(void)
