@@ -35,7 +35,7 @@
 
 #define DEFAULT_AUDIO_SAMPLE_RATE	AUDIO_SAMPLE_RATE_22050
 
-#if defined(PLATFORM_WIN32)
+#if defined(PLATFORM_WINDOWS)
 #define DEFAULT_AUDIO_FRAGMENT_SIZE	AUDIO_FRAGMENT_SIZE_1024
 #else
 #define DEFAULT_AUDIO_FRAGMENT_SIZE	AUDIO_FRAGMENT_SIZE_512
@@ -124,6 +124,7 @@ int getSoundListSize(void);
 int getMusicListSize(void);
 struct FileInfo *getSoundListEntry(int);
 struct FileInfo *getMusicListEntry(int);
+char *getSoundInfoEntryFilename(int);
 char *getMusicInfoEntryFilename(int);
 char *getCurrentlyPlayingMusicFilename(void);
 int getSoundListPropertyMappingSize(void);
@@ -134,8 +135,8 @@ void InitSoundList(struct ConfigInfo *, int, struct ConfigTypeInfo *,
 		   char **, char **, char **, char **, char **);
 void InitMusicList(struct ConfigInfo *, int, struct ConfigTypeInfo *,
 		   char **, char **, char **, char **, char **);
-void InitReloadCustomSounds(char *);
-void InitReloadCustomMusic(char *);
+void InitReloadCustomSounds(void);
+void InitReloadCustomMusic(void);
 void FreeAllSounds(void);
 void FreeAllMusic(void);
 
