@@ -76,7 +76,9 @@
 // font structure definitions
 
 void InitFontInfo(struct FontBitmapInfo *, int,
-		  int (*function1)(int), int (*function2)(char *));
+		  int (*function1)(int),
+                  int (*function2)(char *),
+                  char * (*function3)(int));
 void FreeFontInfo(struct FontBitmapInfo *);
 
 struct FontBitmapInfo *getFontBitmapInfo(int);
@@ -92,19 +94,24 @@ void getFontCharSource(int, char, Bitmap **, int *, int *);
 int maxWordLengthInRequestString(char *);
 
 void DrawInitText(char *, int, int);
+void DrawInitTextHead(char *);
+void DrawInitTextItem(char *);
 
 void DrawTextF(int, int, int, char *, ...);
 void DrawTextFCentered(int, int, char *, ...);
 void DrawTextS(int, int, int, char *);
 void DrawTextSCentered(int, int, char *);
 void DrawTextSAligned(int, int, char *, int, int);
-void DrawTextAligned(int, int, char *, int, int);
 void DrawText(int, int, char *, int);
 void DrawTextExt(DrawBuffer *, int, int, char *, int, int);
 
 char *GetTextBufferFromFile(char *, int);
+int DrawTextArea(int, int, char *, int, int, int, int, int, int,
+		 boolean, boolean, boolean);
 int DrawTextBuffer(int, int, char *, int, int, int, int, int, int,
 		   boolean, boolean, boolean);
+int DrawTextBufferS(int, int, char *, int, int, int, int, int, int,
+		    boolean, boolean, boolean);
 int DrawTextBufferVA(int, int, char *, va_list, int, int, int, int, int, int,
 		     boolean, boolean, boolean);
 int DrawTextFile(int, int, char *, int, int, int, int, int, int,
